﻿using System;
using System.Linq;
using UnityEngine;

namespace ColoredControllers
{
    class ColoredController: MVRScript
    {
        protected JSONStorableBool AsymetricalColorsJSON;

        private FreeControllerV3 _headControl;
        private FreeControllerV3 _neckControl;
        private FreeControllerV3 _rNippleControl;
        private FreeControllerV3 _lNippleControl;
        private FreeControllerV3 _rElbowControl;
        private FreeControllerV3 _lElbowControl;
        private FreeControllerV3 _rKneeControl;
        private FreeControllerV3 _lKneeControl;
        private FreeControllerV3 _lThighControl;
        private FreeControllerV3 _rThighControl;

        private FreeControllerV3 _rShoulderControl;
        private FreeControllerV3 _lShoulderControl;

        private FreeControllerV3 _chestControl;
        private FreeControllerV3 _abdomenControl;
        private FreeControllerV3 _abdomen2Control;
        private FreeControllerV3 _pelvisControl;
        private FreeControllerV3 _rToeControl;
        private FreeControllerV3 _lToeControl;

        private FreeControllerV3 _lhandControl;
        private FreeControllerV3 _rhandControl;
        private FreeControllerV3 _hipControl;
        private FreeControllerV3 _lFootControl;
        private FreeControllerV3 _rFootControl;

        private FreeControllerV3 _lArmControl;
        private FreeControllerV3 _rArmControl;

        // Nodes colors (R,G,B)
        private Color purple = new Color(0.7f, 0.3f, 0.7f, 0.9f);
        private Color darkPurple = new Color(0.337f, 0.101f, 0.325f, 0.5f);
        private Color pink = new Color(1f, 0f, 1f, 0.7f);
        private Color lightPink = new Color(0.886f, 0.713f, 0.862f, 0.8f);
        private Color red = new Color(1f, 0.0f, 0.0f, 0.5f);
        private Color green = new Color(0f, 1f, 0.0f, 0.5f);
        private Color darkGreen = new Color(0f, 0.7f, 0.0f, 0.5f);

        private Color lightBlue = new Color(0.0f, 0.8f, 0.9f, 0.6f);
        private Color blue = new Color(0f, 0f, 1f, 0.4f);
        private Color darkBlue = new Color(0f, 0f, 0.4f, 0.4f);
        private Color VeryDarkBlue = new Color(0f, 0f, 0.3f, 0.6f);

        private Color lightOrange = new Color(1f, 0.8f, 0f, 0.6f);
        private Color orange = new Color(1f, 0.4f, 0f, 0.4f);
        private Color darkOrange = new Color(0.8f, 0.3f, 0f, 0.4f);
        private Color veryDarkOrange = new Color(0.5f, 0.1f, 0f, 0.6f);

        private Color yellow = new Color(1f, 1f, 0.0f, 0.9f);
        private Color lightYellow = new Color(0.988f, 0.968f, 0.670f, 0.6f);
        private Color white = new Color(1f,1f,1f,0.5f);

        private Color darkRed = new Color(0.4f, 0f, 0.0f, 0.2f);

        private bool changeDisabledColored = true; // Set to false if you want to keep the default red color for disabled controller
        private bool separateArmsLegs = false; // Give different color for left/right limbs
        
        private JSONStorableBool asymetricalColorsBool;

        private Atom _person;

        private void SetColors()
        {
            // Add your color here if you want.
            _headControl.onColor = pink;

            _lhandControl.onColor = blue;
            _rhandControl.onColor = blue;

            _lElbowControl.onColor = lightBlue;
            _rElbowControl.onColor = lightBlue;

            _hipControl.onColor = pink;

            _lKneeControl.onColor = darkPurple;
            _rKneeControl.onColor = darkPurple;

            _lFootControl.onColor = orange;
            _rFootControl.onColor = orange;

            if (asymetricalColorsBool.val == true)
            {
                _lhandControl.onColor = lightOrange;
                _lElbowControl.onColor = orange;
                _lKneeControl.onColor = darkOrange;
                _lFootControl.onColor = veryDarkOrange;

                _rhandControl.onColor = lightBlue;
                _rElbowControl.onColor = blue;
                _rKneeControl.onColor = darkBlue;
                _rFootControl.onColor = VeryDarkBlue;
                
            }
        }
        private void UpdateAllControllers()
        {
            UpdateState(_headControl);
            UpdateState(_rShoulderControl);
            UpdateState(_lShoulderControl);
            UpdateState(_lArmControl);
            UpdateState(_rArmControl);
            UpdateState(_lElbowControl);
            UpdateState(_rElbowControl);
            UpdateState(_lhandControl);
            UpdateState(_rhandControl);
            UpdateState(_rNippleControl);
            UpdateState(_lNippleControl);
            UpdateState(_chestControl);
            UpdateState(_abdomenControl);
            UpdateState(_abdomen2Control);
            UpdateState(_pelvisControl);
            UpdateState(_hipControl);
            UpdateState(_lThighControl);
            UpdateState(_rThighControl);
            UpdateState(_lKneeControl);
            UpdateState(_rKneeControl);
            UpdateState(_lFootControl);
            UpdateState(_rFootControl);
            UpdateState(_rToeControl);
            UpdateState(_lToeControl);
        }

        public void ApplyColors()
        {
            // We look for the various controls.
            _headControl = (FreeControllerV3)_person.GetStorableByID("headControl");
            _neckControl = (FreeControllerV3)_person.GetStorableByID("neckControl");

            _rShoulderControl = (FreeControllerV3)_person.GetStorableByID("rShoulderControl");
            _lShoulderControl = (FreeControllerV3)_person.GetStorableByID("lShoulderControl");

            _lArmControl = (FreeControllerV3)_person.GetStorableByID("lArmControl");
            _rArmControl = (FreeControllerV3)_person.GetStorableByID("rArmControl");

            _lElbowControl = (FreeControllerV3)_person.GetStorableByID("lElbowControl");
            _rElbowControl = (FreeControllerV3)_person.GetStorableByID("rElbowControl");

            _lhandControl = (FreeControllerV3)_person.GetStorableByID("lHandControl");
            _rhandControl = (FreeControllerV3)_person.GetStorableByID("rHandControl");

            _rNippleControl = (FreeControllerV3)_person.GetStorableByID("rNippleControl");
            _lNippleControl = (FreeControllerV3)_person.GetStorableByID("lNippleControl");

            _chestControl = (FreeControllerV3)_person.GetStorableByID("chestControl");
            _abdomenControl = (FreeControllerV3)_person.GetStorableByID("abdomenControl");
            _abdomen2Control = (FreeControllerV3)_person.GetStorableByID("abdomen2Control");
            _pelvisControl = (FreeControllerV3)_person.GetStorableByID("pelvisControl");

            _hipControl = (FreeControllerV3)_person.GetStorableByID("hipControl");

            _lThighControl = (FreeControllerV3)_person.GetStorableByID("lThighControl");
            _rThighControl = (FreeControllerV3)_person.GetStorableByID("rThighControl");

            _lKneeControl = (FreeControllerV3)_person.GetStorableByID("lKneeControl");
            _rKneeControl = (FreeControllerV3)_person.GetStorableByID("rKneeControl");

            _lFootControl = (FreeControllerV3)_person.GetStorableByID("lFootControl");
            _rFootControl = (FreeControllerV3)_person.GetStorableByID("rFootControl");

            _rToeControl = (FreeControllerV3)_person.GetStorableByID("rToeControl");
            _lToeControl = (FreeControllerV3)_person.GetStorableByID("lToeControl");

            // Disabble state
            if (changeDisabledColored == true)
            {
                _headControl.offColor = darkRed;
                _neckControl.offColor = darkRed;

                _rShoulderControl.offColor = darkRed;
                _lShoulderControl.offColor = darkRed;

                _lArmControl.offColor = darkRed;
                _rArmControl.offColor = darkRed;

                _lElbowControl.offColor = darkRed;
                _rElbowControl.offColor = darkRed;

                _lhandControl.offColor = darkRed;
                _rhandControl.offColor = darkRed;

                _chestControl.offColor = darkRed;
                _abdomenControl.offColor = darkRed;
                _abdomen2Control.offColor = darkRed;
                _pelvisControl.offColor = darkRed;

                _hipControl.offColor = darkRed;

                _lThighControl.offColor = darkRed;
                _rThighControl.offColor = darkRed;

                _rNippleControl.offColor = darkRed;
                _lNippleControl.offColor = darkRed;

                _lKneeControl.offColor = darkRed;
                _rKneeControl.offColor = darkRed;

                _lFootControl.offColor = darkRed;
                _rFootControl.offColor = darkRed;

                _rToeControl.offColor = darkRed;
                _lToeControl.offColor = darkRed;
            }

            SetColors();
            UpdateAllControllers();
        }

        /*
         * We need to "refresh" the controller so that the color is updater. There is probably a better way to do it, but this seems to work fine.
         * The check here is to avoid enabling controllers that where off.
         */
        private void UpdateState(FreeControllerV3 _ctrl)
        {
            if (_ctrl.currentPositionState == FreeControllerV3.PositionState.Off)
            {
                _ctrl.currentPositionState = FreeControllerV3.PositionState.On;
                _ctrl.currentPositionState = FreeControllerV3.PositionState.Off;
            }
            else
            {
                _ctrl.currentPositionState = FreeControllerV3.PositionState.Off;
                _ctrl.currentPositionState = FreeControllerV3.PositionState.On;
            }
        }

        private JSONStorableBool SetupToggle(string name, bool defaultValue, bool rightSide)
        {
            JSONStorableBool storable = new JSONStorableBool(name, defaultValue);
            storable.storeType = JSONStorableParam.StoreType.Full;
            CreateToggle(storable, rightSide);
            RegisterBool(storable);
            return storable;
        }

        public override void Init()
        {
            try
            {
                SuperController.singleton.onAtomAddedHandlers += OnAtomAdded;
                SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaed;

                asymetricalColorsBool = SetupToggle("Asymetrical Colors", false, false);
                asymetricalColorsBool.setCallbackFunction += (bool v) => { separateArmsLegs = v;  LookingForPersons(); };

                if (containingAtom.type == "Person")
                {
                    SuperController.LogMessage("This should be used as a Session Plugin, but it will still work.");
                    _person = containingAtom;
                    LookingForPersons();
                }

                OnEnable();
            }
            catch(Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }

        }
        private void LookingForPersons()
        {
            foreach (var person in SuperController.singleton.GetAtoms().Where(a => a.type == "Person"))
            {
                _person = person;
                ApplyColors();
            }
        }

        private void OnSceneLoaed()
        {
            LookingForPersons();
        }

        private void OnAtomAdded(Atom a)
        {
            LookingForPersons();
        }

        public void OnEnable()  
        {
            if (containingAtom == null) return;

            if (ReferenceEquals(_person, null))
            {
                LookingForPersons();
            }

        }
    }
}
